import { View, ScrollView, Button } from '@tarojs/components'
import React, { useCallback, memo, useEffect, useMemo, useRef, useState, ReactNode, forwardRef, useImperativeHandle } from 'react'
import styles from "./index.module.scss"
import classnames from "classnames";
import Taro, { usePullDownRefresh, useRouter, useDidShow } from '@tarojs/taro';
import { alert } from '@/common/common'
import { formatPriceDiv, formatDateTime, formatWeightDiv } from '@/common/format'
import IconFont from '@/components/iconfont/iconfont'
import Popup from '@/components/popup';
import { goLink } from '@/common/common'
import { mppurchaserlist, mppurchaserlabeladd } from "@/api/customer"
interface Props {
    showPopup: boolean,
    handClose: () => void,
    purchaser_id?: string | number
    diseShow?: () => void
}
export default memo(forwardRef((props: Props, ref) => {
    const router = useRouter()
    useEffect(() => {
        getList()
    }, [])
    const { fetchData } = mppurchaserlist()
    const getList = async () => {
        const res = await fetchData()
        if (res.data) {
            setModeList([...res.data.list])
        }
    }

    //标签
    const [modeList, setModeList] = useState<any[]>([])
    //选择标签
    const handCheckMode = (item) => {
        modeList.map(it => {
            if (it.id === item.id) {
                it.checked = !it.checked
            }
            return it
        })
        setModeList([...modeList])
    }

    const handReset = () => {
        modeList.map(it => {
            it.checked = false
            return it
        })
        setModeList([...modeList])
    }
    const { fetchData: addFetch } = mppurchaserlabeladd()
    const handSure = async () => {
        const arr: any[] = []
        const fltArr = modeList.filter(item => { return item.checked })
        fltArr.forEach(item => {
            arr.push({
                label_id: item.id
            })
        })
        Taro.showLoading({
            title: '请稍等...',
            mask: true
        })
        const res = await addFetch({ binding_label: arr, purchaser_id: Number(router.params.purchaser_id) })
        if (res.msg == 'success') {
            Taro.showToast({
                icon: 'success',
                title: '成功'
            })
            Taro.hideLoading()
        } else {
            Taro.showToast({
                icon: 'error',
                title: res.msg
            })
            Taro.hideLoading()
        }
        props?.diseShow?.()
    }

    const isDisabled = useMemo(() => {
        let arr = modeList.filter(item => { return item.checked })
        if (arr.length) {
            return false
        } else {
            return true
        }
    }, [modeList])

    const choseNums = useMemo(() => {
        let arr = modeList.filter(item => { return item.checked })
        return arr.length
    }, [modeList])


    useImperativeHandle(ref, () => ({
        modeList,
        setModeList
    }))
    return (
        <Popup title={'选择标签'} show={props.showPopup} onClose={() => { props.handClose() }}>
            <ScrollView scrollY className={styles.scrollView}>
                <View className={styles.thirdBox}>
                    <View className={styles.thirdTopfont}>自定义标签分组</View>
                    <View className={styles.flexModebox}>
                        {modeList.map((item, index) => {
                            return (
                                <View
                                    onClick={() => {
                                        handCheckMode(item)
                                    }}
                                    className={classnames(item.checked ? styles.activemodeBox : styles.modeBox)}
                                    key={index}>
                                    {item.name}
                                </View>
                            )
                        })}
                    </View>
                </View>
            </ScrollView>
            <View className={styles.bottomBox}>
                <Button
                    className={styles.resetBox}
                    onClick={() => {
                        handReset()
                    }}>
                    重置
                </Button>
                <Button
                    className={classnames(isDisabled ? styles.button : styles.activeButton)}
                    disabled={isDisabled}
                    onClick={() => {
                        handSure?.()
                    }}>

                    保存({choseNums})
                </Button>
            </View>
        </Popup>
    )
}))