import { Text, View } from '@tarojs/components'
import { FC, forwardRef, memo, useEffect, useMemo, useRef, useState, useTransition } from 'react'
import styles from './index.module.scss'
import classnames from 'classnames'
import { formatMeterDiv } from '@/common/format'
import LayoutBlock from '@/components/layoutBlock'
import Tag from '@/components/tag'
import Divider from '@/components/divider'
import ColorKindItem from '../colorKindItem'
import { EnumSaleMode } from '@/common/Enumerate'
import { selectList } from '../../config'
import { Goods, GoodsMeta, ShoppingDispatchType, ShoppingStateContextValue, useShoppingDispatch, useShoppingState } from '../../context'
import IconFont from '@/components/iconfont/iconfont'
import { alert, isEmptyObject } from '@/common/common'
import classNames from 'classnames'
import LoadingCard from '@/components/loadingCard'
import { ShoppingCartListApi } from '@/api'
import { ShoppingStore } from '../../context/shoppingStore'
import { usePropsValue } from '@/use/useCommon'

interface ButtonPropsType {
  isActive: boolean
  onClick?: Function
  children?: React.ReactNode
  customStyle?: React.CSSProperties
}
// 订单类型
const SaleModeButton: FC<ButtonPropsType> = props => {
  const { onClick, children, isActive = false, customStyle } = props
  const handleClick = () => {
    onClick?.()
  }
  return (
    <View className={classnames(styles.saleModeButton, isActive && styles['saleModeButton--active'])} style={customStyle} onClick={handleClick}>
      <View className={classnames(styles['saleModeButton--text'])}>{children}</View>
    </View>
  )
}

const DrawerButton = memo<{ isOpen: boolean }>(({ isOpen }) => {
  return (
    <View className={styles.drawerButton}>
      {isOpen ? <IconFont name='icon-shouqi' size={52}></IconFont> : <IconFont name='icon-zhankai' size={52}></IconFont>}
    </View>
  )
})

enum BackEndSaleModeListFieldMap {
  bulk_color_list = 0,
  length_cut_color_list = 1,
  weight_cut_color_list = 2,
}

type PropsType = {
  itemData?: ShoppingCartData
  state?: {
    multipleSelection?: GoodsMeta['multipleSelection']
    currentCheckedPurchaserId?: number
    Observer?: ShoppingStore
  }
}

const ShoppingCartItem: FC<PropsType> = props => {
  const { state } = props
  console.log('props ShoppingCartItem', props)

  const [itemData, setItemData] = usePropsValue({
    value: props.itemData,
    defaultValue: props.itemData,
  })
  const { multipleSelection, currentCheckedPurchaserId } = state!
  const dispatch = useShoppingDispatch()

  const [openDetail, setOpenDetail] = useState(false)

  const handleOpenDetail = () => {
    setOpenDetail(isOpen => !isOpen)
    handleClickLayout()
  }

  const [selected, setSelect] = useState<EnumSaleMode>(0)

  const onSelectOrderType = (type: EnumSaleMode) => {
    if (isPending) return
    setSelect(type)
    dispatch({ type: ShoppingDispatchType.UPDATE_CURRENT_CHECKED_SALEMODE, data: type })
    // 重置预估金额
    dispatch({ type: ShoppingDispatchType.UPDATE_SELECTED_AMOUNT, data: 0 })
    dispatch({ type: ShoppingDispatchType.UPDATE_MULTIPLE_SELECTION_STATUS, data: false })
  }


  const handleClickLayout = () => {
    if (currentCheckedPurchaserId === itemData?.purchaser_id) {
      return
    }
    dispatch({ type: ShoppingDispatchType.UPDATE_SELECTED_AMOUNT, data: 0 })
    dispatch({ type: ShoppingDispatchType.UPDATE_CURRENT_CHECKED_SALEMODE, data: selected })
    dispatch({
      type: ShoppingDispatchType.UPDATE_CURRENT_CHECKED_PURCHASERID,
      data: itemData?.purchaser_id as number,
    })
    dispatch({ type: ShoppingDispatchType.UPDATE_MULTIPLE_SELECTION_STATUS, data: false })
  }

  // 统计已选面料
  const materialChecked = useMemo(() => {
    if (!multipleSelection || isEmptyObject(multipleSelection)) return 0
    return new Set(
      Object.values(multipleSelection)?.reduce((prev, item: Goods) => {
        if (item.sale_mode === selected) {
          return [...prev, item.product_code]
        }
        return prev
      }, []),
    ).size
  }, [multipleSelection, currentCheckedPurchaserId, selected, itemData])

  // 统计已选颜色
  const colorChecked = useMemo(() => {
    console.log('colorChecked item',multipleSelection);
    if (!multipleSelection || isEmptyObject(multipleSelection)) return 0
    return new Set(
      Object.values(multipleSelection).reduce((prev, item: Goods) => {
        if (item.sale_mode === selected) {
          return [...prev, item.product_color_code]
        }
        return prev
      }, []),
    ).size
  }, [multipleSelection, currentCheckedPurchaserId, selected, itemData])

  // 统计已选条数 / 米数
  const lengthOrRollChecked = useMemo(() => {
    console.log('lengthOrRollChecked item',multipleSelection);
    if (!multipleSelection || isEmptyObject(multipleSelection)) return 0
    return (
      Object.values(multipleSelection).reduce((prev, item: Goods) => {
        return prev + item.count
      }, 0) || 0
    )
  }, [multipleSelection, currentCheckedPurchaserId, selected, itemData])

  const [isPending, startTransition] = useTransition()
  const { fetchData } = ShoppingCartListApi()

  // 发布订阅

  useEffect(() => {
    console.log('update multipleSelection',multipleSelection);
    const unsubscribe = state?.Observer?.subscribe(async id => {
      if (itemData?.purchaser_id !== id) return
      console.log('request new data start run')
      const res = await fetchData({
        purchaser_id: id,
      })
      console.log('res===>', res)
      if (res.success) {
        // 转成 对象形式
        // 4562: {
        //  id: 4562,
        //    ...
        // }
        const newGoodsKind = Object.fromEntries(
          res.data[0]?.[BackEndSaleModeListFieldMap[selected]].map(item => [
            item?.id,
            {
              id: item?.id,
              estimate_amount: item.estimate_amount,
              product_code: item.product_code,
              product_color_code: item.product_color_code,
              sale_mode: item.sale_mode,
              count: selected === EnumSaleMode.Bulk ? item.roll : Number(formatMeterDiv(item.length)),
            },
          ]),
        )
        dispatch({
          type: ShoppingDispatchType.UPDATE_CHANGED_CHECKBOX,
          data: {
            purchaserId: id,
            goodsKind: newGoodsKind,
            multipleSelection: Object.fromEntries(
              Object.keys(multipleSelection!)
                .map(id => [
                  id,
                  {
                    id,
                    estimate_amount: newGoodsKind[id].estimate_amount,
                    product_code: newGoodsKind[id].product_code,
                    product_color_code: newGoodsKind[id].product_color_code,
                    sale_mode: newGoodsKind[id].sale_mode,
                    count: newGoodsKind[id].count,
                  },
                ]),
            ),
          },
        })
        setItemData(() => res.data[0])
      }else{
        alert.error('请求失败')
      }
    })
    // 取消订阅
    return unsubscribe
  }, [multipleSelection])

  return (
    <LayoutBlock
      circle
      customClassName={classnames(styles.layout, itemData?.purchaser_id === currentCheckedPurchaserId ? styles.selected : '')}
      onClick={handleClickLayout}>
      <View className={classNames(styles.checkbox, 'flex-row', 'justify-between')} onClick={handleOpenDetail}>
        <View className={styles.topItem}>
          <View className='flex-row items-center'>
            <View className={styles.topTitle}>{itemData?.purchaser_name}</View>
            <Tag type='info' size='normal' circle customStyle={{ backgroundColor: '#f0f0f0', borderColor: 'transparent' }} plain>
              {itemData?.sale_user_name}
            </Tag>
          </View>
          <View className={styles.summary}>
            <Text>
              已选 {materialChecked} 种面料，{colorChecked} 个颜色，共{' '}
              {selected === EnumSaleMode.Bulk ? `${lengthOrRollChecked} 条` : `${(lengthOrRollChecked).toLocaleString()} 米`}
            </Text>
          </View>
        </View>
        <DrawerButton isOpen={openDetail} />
      </View>
      {/* 减少节点渲染 */}
      {openDetail && (
        <>
          <View className={classnames(styles.detailBox, openDetail ? styles.drawerOpen : styles.drawerClose)}>
            <View className={styles.checkbox}>
              <Divider customClassName={styles.line}></Divider>
            </View>
            <View className={styles.orderType}>
              <View className={styles.orderTitle}>订单类型</View>
              <View className={styles.orderTypeDetail}>
                <SaleModeButton
                  customStyle={{ padding: '0 50rpx' }}
                  isActive={selected === EnumSaleMode.Bulk}
                  onClick={() => onSelectOrderType(EnumSaleMode.Bulk)}>
                  大货
                </SaleModeButton>
                <SaleModeButton
                  customStyle={{ padding: '0 50rpx' }}
                  isActive={selected === EnumSaleMode.Plate}
                  onClick={() => onSelectOrderType(EnumSaleMode.Plate)}>
                  剪板
                </SaleModeButton>
                <SaleModeButton
                  customStyle={{ padding: '0 50rpx' }}
                  isActive={selected === EnumSaleMode.BulkCut}
                  onClick={() => onSelectOrderType(EnumSaleMode.BulkCut)}>
                  散剪
                </SaleModeButton>
              </View>
            </View>
            <View className={styles.orderColor}>
              <Divider customClassName={styles.line}></Divider>
              <View className={classnames(styles.orderTitle, 'justify-between')}>
                <Text>布料颜色</Text>
                <Tag type='danger' size='normal' circle plain>
                  {selectList[selected].title}单位：{selectList[selected].unit}
                </Tag>
              </View>
            </View>
            <View className={styles.orderContainer}>
              <GoodsList
                multipleSelection={multipleSelection}
                itemData={itemData}
                selected={selected}
                isPending={isPending}
                startTransition={startTransition}
              />
            </View>
          </View>
        </>
      )}
    </LayoutBlock>
  )
}

interface GoodsListPropType {
  itemData?: ShoppingCartData
  multipleSelection?: GoodsMeta['multipleSelection']
  selected: EnumSaleMode
  isPending: boolean
  startTransition: React.TransitionStartFunction
}
const GoodsList = memo<GoodsListPropType>(props => {
  const { itemData, selected, isPending, startTransition, multipleSelection } = props

  const currentSelected = useRef<EnumSaleMode | null>(null)

  const dispatch = useShoppingDispatch()

  const [component, setComponent] = useState<JSX.Element | null>(null)

  // 更新 GoodsList 组件
  const updateComponent = () => {
    setComponent(
      itemData?.[BackEndSaleModeListFieldMap[selected]].length !== 0 ? (
        itemData?.[BackEndSaleModeListFieldMap[selected]].map(item => {
          console.log('item===>', item)
          return <ColorKindItem purchaserId={itemData.purchaser_id} key={item.id} itemData={item}></ColorKindItem>
        })
      ) : (
        <View className={styles.noList}>暂无数据</View>
      ),
    )
  }

  useEffect(() => {
    const newGoodsKind = Object.fromEntries(
      itemData?.[BackEndSaleModeListFieldMap[selected]].map(item => [
        item?.id,
        {
          id: item?.id,
          estimate_amount: item.estimate_amount,
          product_code: item.product_code,
          product_color_code: item.product_color_code,
          sale_mode: item.sale_mode,
          count: selected === EnumSaleMode.Bulk ? item.roll : Number(formatMeterDiv(item.length)),
        },
      ]),
    )
    // 这里做一层比较是为了 重新渲染的时候如果没有切换订单类型的话就不让面料的选中状态初始化
    if (currentSelected.current === selected) {
      dispatch({
        type: ShoppingDispatchType.UPDATE_CHANGED_CHECKBOX,
        data: {
          purchaserId: itemData?.purchaser_id!,
          goodsKind: newGoodsKind,
          multipleSelection: multipleSelection,
        },
      })
      updateComponent()
    } else {
      // 重新把当前的选中状态赋值给ref 作为下一次比较的旧状态
      currentSelected.current = selected
      dispatch({
        type: ShoppingDispatchType.UPDATE_CHANGED_CHECKBOX,
        data: {
          purchaserId: itemData?.purchaser_id!,
          goodsKind: newGoodsKind,
          multipleSelection: {},
        },
      })
      startTransition(updateComponent)
    }
  }, [itemData, selected, multipleSelection])

  return (
    <>
      {isPending ? (
        <View className={styles.noList} style={{margin: '60rpx 0'}}>
          <LoadingCard />
        </View>
      ) : (
        component
      )}
    </>
  )
})
// State 分割组件 思路就是把 context 直接通过 props 的形式传给组件，这样的话就解决了 context 强制刷新 memo 的问题了
// 那么当 context 内的 value 被更新的时候，react 只会强制渲染 Wrapper
const withStateSlice = (comp, slice) => {
  const MemoComp = memo(comp, (prevProps, nextProps) => {
    let needMemo = true
    // const prevCheckedPurchaserId = prevProps.state.currentCheckedPurchaserId
    // const nextCheckedPurchaserId = nextProps.state.currentCheckedPurchaserId
    // const purchaser_id = prevProps.itemData.purchaser_id
    // const prevMultipleSelection = prevProps.state.colorStore[purchaser_id].multipleSelection
    // const nextMultipleSelection = nextProps.state.colorStore[purchaser_id].multipleSelection
    if (JSON.stringify(prevProps.itemData) !== JSON.stringify(nextProps.itemData)) {
      needMemo = false
    }
    if(prevProps.itemData.purchaser_name === 'Hhjh'){
      console.log('------withStateSlice props-------');
      console.log('withStateSlice props prevProps', prevProps);
      console.log('withStateSlice props nextProps', nextProps);
      console.log('------withStateSlice props-------');
    }
    if (JSON.stringify(prevProps.state) !== JSON.stringify(nextProps.state)) {
      console.log('MultipleSelection 有变化');
      needMemo = false
    }
    // if (JSON.stringify(prevCheckedPurchaserId) !== JSON.stringify(nextCheckedPurchaserId)) {
    //   console.log('checkedPurchaserId 有变化');
    //   needMemo = false
    // }
    return needMemo
  })
  const Wrapper = (props, ref) => {
    const state = useShoppingState()
    return <MemoComp ref={ref} state={slice(state, props)} {...props} />
  }
  return memo(forwardRef(Wrapper))
}

const ShoppingCartItemWithStateSlice = withStateSlice(ShoppingCartItem, (state: ShoppingStateContextValue, props) => ({
  multipleSelection: state.colorStore?.[props.itemData?.purchaser_id]?.['multipleSelection'],
  currentCheckedPurchaserId: state.currentCheckedPurchaserId,
  Observer: state.Observer,
}))

export default ShoppingCartItemWithStateSlice
